﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using AtmoRemote;
using System.Timers;
using System.Data;
using System.Data.SqlClient;
using CelsiusNet;
using System.ComponentModel;


namespace AtmoRemoteGui
{


    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Device device;   
        

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
			//This creates the device and establishes the connection to the chamber 
			//with the specified IP address.
            //The lower protocol is http, the commands are send via webrequest.
            //Standard timeout of an http webrequest is 25 seconds
            try
            {
                device = Device.CreateDevice(txtIp.Text);
            }
            catch (Exception ex)
            {
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
            }
            
            //If there is no connection device is null
            if (device == null)
			{
				AddMessageToStatusTextBox("Device not available");
				return;
			}
            else
            {
                btnStartConnection.Content = "Connected";
            }

			//If Remote Control is not activated on chamber
			if (device.DataElements.RemoteState == (int)CommunicationDirection.None)
				AddMessageToStatusTextBox("Remote is not activated on chamber");

			//This updates the listView with all available data
            UpdateListView();
        }

        private void UpdateListView()
        {
            listView.Items.Clear();

			// GetListOfActivePropertyNames() receives a list of string elements 
			// which represents different properties of the chamber
            foreach (string element in device.DataElements.GetListOfActivePropertyNames())
            {
                DataElement data = device.DataElements.GetDataElementFromPropertyName(element);
                listView.Items.Add(new MyItem { Name = element, Value = data.ToString() });
            }
        }

        public class MyItem
        {
            public string Name { get; set; }
            public string Value { get; set; }
        }

        private void btnStartProfile_Click(object sender, RoutedEventArgs e)
        {
            // Configure open file dialog box
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = "Profile"; // Default file name
            dlg.DefaultExt = ".atpro"; // Default file extension
            dlg.Filter = "Profiles (.atpro)|*.atpro"; // Filter files by extension

            // Show open file dialog box
            Nullable<bool> fileIsValid = dlg.ShowDialog();

            // Process open file dialog box results
            if (!fileIsValid.Value)
                return;

			try
			{
				//StartProgram() uploads the given file to the chamber and 
				//directly starts the profile.
				AddMessageToStatusTextBox(device.StartProgam(dlg.FileName));
				device.UpdateData();
                UpdateListView();
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
			}
            
        }

        private void btnStopProfile_Click(object sender, RoutedEventArgs e)
        {
			try
			{
				//When a profile is running you can stop it by calling StopProgram(),
				//Cabinet will get into idle mode
				device.StopProgam();
                device.UpdateData();
				UpdateListView();
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
			}  
        }


		private void btnDeactivateProfile_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				//When a profile is running or chamber is in idle mode you can stop 
				//it by calling DeactivateProgram(), cabinet will get into manual mode
				device.DeactivateProgam();
                device.UpdateData();
                UpdateListView();
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
			}
		}

        private void btnRead_Click(object sender, RoutedEventArgs e)
        {
			try
			{
				//UpdateData() starts a communication. When a parameter was set before
				//it is going to be set.
				AddMessageToStatusTextBox(device.UpdateData());
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
			}

			UpdateListView();
        }

		private void AddMessageToStatusTextBox(string text)
		{
			txtStatus.Clear();
			txtStatus.Text = text;
			txtStatus.ScrollToEnd();
		}

		private void AddMessageToStatusTextBox(List<StatusMessage> statusMessages)
		{
			
			string stringToAdd = "";

			foreach (StatusMessage message in statusMessages)
				stringToAdd += message.Message + "\n";

			AddMessageToStatusTextBox(stringToAdd);
		}

        private void ButtonSetValue_Click(object sender, RoutedEventArgs e)
        {
            if (listView.SelectedIndex == -1)
			{
				AddMessageToStatusTextBox("Nothing selected");
				return;
			}

			if (txtWrite.Text == "")
			{
				AddMessageToStatusTextBox("No value");
				return;
			}

            MyItem item = (MyItem)listView.SelectedItem;
            item.Value = txtWrite.Text;

			try
			{
				//The property name which was read on the beginning has been selected
				//The value was written in the textbox. This property will now be localy 
				//set to the given parameter. This value is yet not set on the cabinet!
				//It will be set when UpdateData() will be called.
				device.DataElements.SetDataElementFromPropertyName(item.Name, item.Value);
				txtWrite.Clear();
				txtSetValueAnswer.Text = item.Name + " " + "=" + " " + item.Value;
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
			}
        }

		private void btnGetProfiles_Click(object sender, RoutedEventArgs e)
		{
			lbPrograms.Items.Clear();
			List<string> programList = new List<string>();

			try
			{
				//This call receives all available profiles on the cabinet.
				programList = device.DownloadProgramList();
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
				return;
			}

			foreach (string element in programList)
			{
				lbPrograms.Items.Add(element.Replace(".atpro", ""));
			}
		}

		private void btnStartSelectedProfile_Click(object sender, RoutedEventArgs e)
		{
			if (lbPrograms.SelectedIndex == -1)
			{
				txtStatus.Text = txtStatus.Text + "No Profile selected";
				return;
			}
            
            //Update current data of chamber
            device.UpdateData();

            ////If program is running give an error message
            //if (device.DataElements.Operation != CelsiusNet.OperationMode.Manual)
            //{
            //    AddMessageToStatusTextBox("Program is active, please stop first");
            //    return;
            //}

			//This does not upload any profiles. It just starts the program
			//which was given by name. If the profile is not available on the cabinet
			//nothing happens.
			AddMessageToStatusTextBox(device.StartProgramOnDevice(lbPrograms.SelectedItem.ToString()));

            device.UpdateData();
            UpdateListView();
			
		}

		private void btnSyncTime_Click(object sender, RoutedEventArgs e)
		{
			try
			{
				//SynchronizeTime() updates directly 4 properties on the cabinet
				//TimeZone, Daylightsavings Date and Time which were taken from 
				//the windows system.
				AddMessageToStatusTextBox(device.SynchronizeTime());
			}
			catch (Exception ex)
			{
				AddMessageToStatusTextBox(ex.Message.ToString());
			}
		}
    }
}
